#ifndef CSG_INFO_H
#define CSG_INFO_H

/** \file
 *  \brief   CSG Rendering Runtime Info
 *  \ingroup CSG
 *  \author  Nigel Stewart (nigels@nigels.com)
 *  \author  RMIT School of Computer Science and Information Technology
 *  \author  RMIT Department of Mechanical and Manufacturing Engineering
 */

#include <csg/summary.h>
#include <misc/timer.h>

class CsgInfo
{
public:
    CsgInfo();
    ~CsgInfo();

    void reset();

    void startFrame();
    void endFrame();

    std::string str() const;

    // Summaries

    summary<int> subtractions;
    summary<int> intersections;
    summary<int> depthComplexity;
    summary<int> depthComplexityFront;
    summary<int> depthComplexityBack;

    summary<unsigned int> stencilBufferKBits;

    summary<unsigned int> depthBits;
    summary<unsigned int> depthClears;
    summary<unsigned int> depthReads;
    summary<unsigned int> depthWrites;

    summary<unsigned int> stencilBits;
    summary<unsigned int> stencilClears;
    summary<unsigned int> stencilReads;
    summary<unsigned int> stencilWrites;

    summary<clock_t> encodeTime;
    summary<clock_t> drawTime;
};

#endif

