//
// Benchmark of glCopyPixels for GL_COLOR,GL_STENCIL,GL_DEPTH
//
// System Setup:
//  Windows NT 3.51, Pentium 100, 32Meg RAM
//  3Demon SX88, Display Driver 2.0, OpenGL driver 1.0.9
//  Visual C++ V4.0
//
// Sample Output:
//
// (500x500)
// Color rate  : 1.5625e+007
// Depth rate  : 3.90625e+006
// Stencil rate: 3.90625e+006
//
// (1024x745)
// Color rate  : 1.12188e+007
// Depth rate  : 3.97333e+006
// Stencil rate: 4.04926e+006
//
// (554x115)
// Color rate  : 1.59275e+007
// Depth rate  : 3.98187e+006
// Stencil rate: 3.98187e+006
//
// Prepared by Nigel Stewart, RMIT University, 1999-2003
// Email: nigels@nigels.com
//

#include <glt/gl.h>
#include <glt/color.h>
#include <glt/info.h>
#include <glt/rgb.h>

#include <misc/timer.h>

#include <glutm/main.h>
#include <glutm/window.h>

#include <iostream>
using namespace std;

static GltString banner =
        "OpenGL Buffer Copying Benchmark\n" \
        "(C) 1999-2003 Nigel Stewart    \n" \
        "http://www.nigels.com/research/\n";

class OpenGLCopySpeed : public GlutWindow
{
public:
    OpenGLCopySpeed();
   ~OpenGLCopySpeed();

   void OnOpen();
   void OnClose();
   void OnDisplay();
};

OpenGLCopySpeed::OpenGLCopySpeed()
: GlutWindow("OpenGL Buffer Copying Benchmark")
{
}

OpenGLCopySpeed::~OpenGLCopySpeed()
{
}

void
OpenGLCopySpeed::OnOpen()
{
    GlutMenu &save = _leftButtonMenu.subMenu();
    save.addEntry("RGB"     ,MENU_CAPTURE_RGB);
    save.addEntry("z-buffer",MENU_CAPTURE_ZBUFFER);

    _leftButtonMenu.addEntry("Save PPM",save);
    _leftButtonMenu.addEntry("Close" ,MENU_CLOSE);
    _leftButtonMenu.attach();

    glEnable(GL_DEPTH_TEST);
    glClearDepth(1.0);
    glTranslatef(0.0,0.0,-1.0);

    cout << banner << endl << GltInfo() << endl;
    cout << "width\theight\tcolour\t\tdepth\t\tstencil" << endl;
}

void
OpenGLCopySpeed::OnClose()
{
    _leftButtonMenu.reset();
}

const GLdouble maximumSamplePeriod = 0.3F;
const GLdouble Mill = 1.0/1000000.0;

void
OpenGLCopySpeed::OnDisplay()
{
    GLuint viewport[4];
    GLuint width,height;

    glGetIntegerv(GL_VIEWPORT,(GLint *) viewport);
    width = viewport[2];
    height = viewport[3];

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT );

    glViewport(0,0,width>>1,height>>1);
    red.glColor();
    glDepthFunc(GL_LESS);
    glutSolidSphere(1.0,10,10);

    glViewport(0,0,width,height);
    glRasterPos2i(0,0);

    cout << width << '\t' << height << '\t';

    {
        GLuint repetitions = 0;
        Timer timer;

        while (1)
        {
            glCopyPixels(0,0,width>>1,height>>1,GL_COLOR);
            glFinish();

            repetitions++;

            if (timer.elapsed()>maximumSamplePeriod) break;
        }

        GLdouble rate = 0.25*repetitions*width*height/timer.elapsed();
        cout << repetitions << " ";
        cout << rate*Mill << "M \t";
    }

    {
        GLuint repetitions = 0;
        Timer timer;

        blue.glColor();
        glDepthFunc(GL_NOTEQUAL);

        while (1)
        {
            glCopyPixels(0,0,width>>1,height>>1,GL_DEPTH);
            glFinish();

            repetitions++;

            if (timer.elapsed()>maximumSamplePeriod) break;
        }

        GLdouble rate = 0.25*repetitions*width*height/timer.elapsed();
        cout << repetitions << " ";
        cout << rate*Mill << "M \t";
    }

    {
        GLuint repetitions = 0;
        Timer timer;

        while (1)
        {
            glCopyPixels(0,0,width>>1,height>>1,GL_STENCIL);
            glFinish();

            repetitions++;

            if (timer.elapsed()>maximumSamplePeriod) break;
        }

        GLdouble rate = 0.25*repetitions*width*height/timer.elapsed();
        cout << repetitions << " ";
        cout << rate*Mill << 'M' << endl;
    }
}

bool GlutMain(const vector<string> &arg)
{
    GlutWindow *main = new OpenGLCopySpeed();
    main->open();
    return true;
}
